@@ -55,6 +55,7 @@ extern "C" {
 #include "searcher.h"
 #include "search_renderer.h"
 #include "opds_dumper.h"
+#include "i18n.h"
 
 #include <zim/uuid.h>
 #include <zim/error.h>
@@ -77,7 +78,6 @@ extern "C" {
 #include "response.h"
 
 #define MAX_SEARCH_LEN 140
-#define KIWIX_MIN_CONTENT_SIZE_TO_DEFLATE 100
 #define DEFAULT_CACHE_SIZE 2
 
 namespace kiwix {
@@ -95,35 +95,197 @@ inline std::string normalizeRootUrl(std::string rootUrl)
   return rootUrl.empty() ? rootUrl : "/" + rootUrl;
 }
 
-// Returns the value of env var `name` if found, otherwise returns defaultVal
-unsigned int getCacheLength(const char* name, unsigned int defaultVal) {
-  try {
-    const char* envString = std::getenv(name);
-    if (envString == nullptr) {
-      throw std::runtime_error("Environment variable not set");
-    }
-    return extractFromString<unsigned int>(envString);
-  } catch (...) {}
-
-  return defaultVal;
+Filter get_search_filter(const RequestContext& request, const std::string& prefix="")
+{
+    auto filter = kiwix::Filter().valid(true).local(true);
+    try {
+      filter.query(request.get_argument(prefix+"q"));
+    } catch (const std::out_of_range&) {}
+    try {
+      filter.maxSize(request.get_argument<unsigned long>(prefix+"maxsize"));
+    } catch (...) {}
+    try {
+      filter.name(request.get_argument(prefix+"name"));
+    } catch (const std::out_of_range&) {}
+    try {
+      filter.category(request.get_argument(prefix+"category"));
+    } catch (const std::out_of_range&) {}
+    try {
+      filter.lang(request.get_argument(prefix+"lang"));
+    } catch (const std::out_of_range&) {}
+    try {
+      filter.acceptTags(kiwix::split(request.get_argument(prefix+"tag"), ";"));
+    } catch (...) {}
+    try {
+      filter.rejectTags(kiwix::split(request.get_argument(prefix+"notag"), ";"));
+    } catch (...) {}
+    return filter;
 }
+
+template<class T>
+std::vector<T> subrange(const std::vector<T>& v, size_t s, size_t n)
+{
+  const size_t e = std::min(v.size(), s+n);
+  return std::vector<T>(v.begin()+std::min(v.size(), s), v.begin()+e);
+}
+
+std::string renderUrl(const std::string& root, const std::string& urlTemplate)
+{
+  MustacheData data;
+  data.set("root", root);
+  auto url = kainjow::mustache::mustache(urlTemplate).render(data);
+  if ( url.back() == '\n' )
+    url.pop_back();
+  return url;
+}
+
+std::string makeFulltextSearchSuggestion(const std::string& lang, const std::string& queryString)
+{
+  return i18n::expandParameterizedString(lang, "suggest-full-text-search",
+               {
+                  {"SEARCH_TERMS", queryString}
+               }
+         );
+}
+
+ParameterizedMessage noSuchBookErrorMsg(const std::string& bookName)
+{
+  return ParameterizedMessage("no-such-book", { {"BOOK_NAME", bookName} });
+}
+
+ParameterizedMessage invalidRawAccessMsg(const std::string& dt)
+{
+  return ParameterizedMessage("invalid-raw-data-type", { {"DATATYPE", dt} });
+}
+
+ParameterizedMessage noValueForArgMsg(const std::string& argument)
+{
+  return ParameterizedMessage("no-value-for-arg", { {"ARGUMENT", argument} });
+}
+
+ParameterizedMessage rawEntryNotFoundMsg(const std::string& dt, const std::string& entry)
+{
+  return ParameterizedMessage("raw-entry-not-found",
+                              {
+                                {"DATATYPE", dt},
+                                {"ENTRY", entry},
+                              }
+  );
+}
+
+ParameterizedMessage tooManyBooksMsg(size_t nbBooks, size_t limit)
+{
+  return ParameterizedMessage("too-many-books",
+                              {
+                                {"NB_BOOKS", beautifyInteger(nbBooks)},
+                                {"LIMIT", beautifyInteger(limit)},
+                              }
+  );
+}
+
+ParameterizedMessage nonParameterizedMessage(const std::string& msgId)
+{
+  const ParameterizedMessage::Parameters noParams;
+  return ParameterizedMessage(msgId, noParams);
+}
+
+struct Error : public std::runtime_error {
+  explicit Error(const ParameterizedMessage& message)
+    : std::runtime_error("Error while handling request"),
+      _message(message)
+  {}
+
+  const ParameterizedMessage& message() const
+  {
+    return _message;
+  }
+
+  const ParameterizedMessage _message;
+};
+
+void checkBookNumber(const Library::BookIdSet& bookIds, size_t limit) {
+  if (bookIds.empty()) {
+    throw Error(nonParameterizedMessage("no-book-found"));
+  }
+  if (limit > 0 && bookIds.size() > limit) {
+    throw Error(tooManyBooksMsg(bookIds.size(), limit));
+  }
+}
+
 } // unnamed namespace
 
-SearchInfo::SearchInfo(const std::string& pattern)
-  : pattern(pattern),
-    geoQuery()
-{}
-
-SearchInfo::SearchInfo(const std::string& pattern, GeoQuery geoQuery)
-  : pattern(pattern),
-    geoQuery(geoQuery)
-{}
-
-SearchInfo::SearchInfo(const RequestContext& request)
-  : pattern(request.get_optional_param<std::string>("pattern", "")),
-    geoQuery(),
-    bookName(request.get_optional_param<std::string>("content", ""))
+std::pair<std::string, Library::BookIdSet> InternalServer::selectBooks(const RequestContext& request) const
 {
+  // Try old API
+  try {
+    auto bookName = request.get_argument("content");
+    try {
+      const auto bookIds = Library::BookIdSet{mp_nameMapper->getIdForName(bookName)};
+      const auto queryString = request.get_query([&](const std::string& key){return key == "content";}, true);
+      return {queryString, bookIds};
+    } catch (const std::out_of_range&) {
+      throw Error(noSuchBookErrorMsg(bookName));
+    }
+  } catch(const std::out_of_range&) {
+    // We've catch the out_of_range of get_argument
+    // continue
+  }
+
+  // Does user directly gives us ids ?
+  try {
+    auto id_vec = request.get_arguments("books.id");
+    if (id_vec.empty()) {
+      throw Error(noValueForArgMsg("books.id"));
+    }
+    for(const auto& bookId: id_vec) {
+      try {
+        // This is a silly way to check that bookId exists
+        mp_nameMapper->getNameForId(bookId);
+      } catch (const std::out_of_range&) {
+        throw Error(noSuchBookErrorMsg(bookId));
+      }
+    }
+    const auto bookIds = Library::BookIdSet(id_vec.begin(), id_vec.end());
+    const auto queryString = request.get_query([&](const std::string& key){return key == "books.id";}, true);
+    return {queryString, bookIds};
+  } catch(const std::out_of_range&) {}
+
+  // Use the names
+  try {
+    auto name_vec = request.get_arguments("books.name");
+    if (name_vec.empty()) {
+      throw Error(noValueForArgMsg("books.name"));
+    }
+    Library::BookIdSet bookIds;
+    for(const auto& bookName: name_vec) {
+      try {
+        bookIds.insert(mp_nameMapper->getIdForName(bookName));
+      } catch(const std::out_of_range&) {
+        throw Error(noSuchBookErrorMsg(bookName));
+      }
+    }
+    const auto queryString = request.get_query([&](const std::string& key){return key == "books.name";}, true);
+    return {queryString, bookIds};
+  } catch(const std::out_of_range&) {}
+
+  // Check for filtering
+  Filter filter = get_search_filter(request, "books.filter.");
+  auto id_vec = mp_library->filter(filter);
+  if (id_vec.empty()) {
+    throw Error(nonParameterizedMessage("no-book-found"));
+  }
+  const auto bookIds = Library::BookIdSet(id_vec.begin(), id_vec.end());
+  const auto queryString = request.get_query([&](const std::string& key){return startsWith(key, "books.filter.");}, true);
+  return {queryString, bookIds};
+}
+
+SearchInfo InternalServer::getSearchInfo(const RequestContext& request) const
+{
+  auto bookIds = selectBooks(request);
+  checkBookNumber(bookIds.second, m_multizimSearchLimit);
+  auto pattern = request.get_optional_param<std::string>("pattern", "");
+  GeoQuery geoQuery;
+
   /* Retrive geo search */
   try {
     auto latitude = request.get_argument<float>("latitude");
@@ -134,10 +296,19 @@ SearchInfo::SearchInfo(const RequestContext& request)
     catch(const std::invalid_argument&) {}
 
   if (!geoQuery && pattern.empty()) {
-    throw std::invalid_argument("No query provided.");
+    throw Error(nonParameterizedMessage("no-query"));
   }
+
+  return SearchInfo(pattern, geoQuery, bookIds.second, bookIds.first);
 }
 
+SearchInfo::SearchInfo(const std::string& pattern, GeoQuery geoQuery, const Library::BookIdSet& bookIds, const std::string& bookFilterQuery)
+  : pattern(pattern),
+    geoQuery(geoQuery),
+    bookIds(bookIds),
+    bookFilterQuery(bookFilterQuery)
+{}
+
 zim::Query SearchInfo::getZimQuery(bool verbose) const {
   zim::Query query;
   if (verbose) {
@@ -175,6 +346,7 @@ InternalServer::InternalServer(Library* library,
                                int port,
                                std::string root,
                                int nbThreads,
+                               unsigned int multizimSearchLimit,
                                bool verbose,
                                bool withTaskbar,
                                bool withLibraryButton,
@@ -185,6 +357,7 @@ InternalServer::InternalServer(Library* library,
   m_port(port),
   m_root(normalizeRootUrl(root)),
   m_nbThreads(nbThreads),
+  m_multizimSearchLimit(multizimSearchLimit),
   m_verbose(verbose),
   m_withTaskbar(withTaskbar),
   m_withLibraryButton(withLibraryButton),
@@ -194,9 +367,8 @@ InternalServer::InternalServer(Library* library,
   mp_daemon(nullptr),
   mp_library(library),
   mp_nameMapper(nameMapper ? nameMapper : &defaultNameMapper),
-  searcherCache(getCacheLength("SEARCHER_CACHE_SIZE", std::max((unsigned int) (mp_library->getBookCount(true, true)*0.1), 1U))),
-  searchCache(getCacheLength("SEARCH_CACHE_SIZE", DEFAULT_CACHE_SIZE)),
-  suggestionSearcherCache(getCacheLength("SUGGESTION_SEARCHER_CACHE_SIZE", std::max((unsigned int) (mp_library->getBookCount(true, true)*0.1), 1U)))
+  searchCache(getEnvVar<int>("KIWIX_SEARCH_CACHE_SIZE", DEFAULT_CACHE_SIZE)),
+  suggestionSearcherCache(getEnvVar<int>("KIWIX_SUGGESTION_SEARCHER_CACHE_SIZE", std::max((unsigned int) (mp_library->getBookCount(true, true)*0.1), 1U)))
 {}
 
 bool InternalServer::start() {
@@ -327,7 +499,7 @@ std::unique_ptr<Response> InternalServer::handle_request(const RequestContext& r
 {
   try {
     if (! request.is_valid_url()) {
-      return HTTP404HtmlResponse(*this, request)
+      return HTTP404Response(*this, request)
              + urlNotFoundMsg;
     }
 
@@ -347,6 +519,14 @@ std::unique_ptr<Response> InternalServer::handle_request(const RequestContext& r
     if (request.get_url() == "/search")
       return handle_search(request);
 
+    if (request.get_url() == "/search/searchdescription.xml") {
+      return ContentResponse::build(
+        *this,
+        RESOURCE::ft_opensearchdescription_xml,
+        get_default_data(),
+        "application/opensearchdescription+xml");
+    }
+
     if (request.get_url() == "/suggest")
      return handle_suggest(request);
 
@@ -359,11 +539,11 @@ std::unique_ptr<Response> InternalServer::handle_request(const RequestContext& r
     return handle_content(request);
   } catch (std::exception& e) {
     fprintf(stderr, "===== Unhandled error : %s\n", e.what());
-    return HTTP500HtmlResponse(*this, request)
+    return HTTP500Response(*this, request)
          + e.what();
   } catch (...) {
     fprintf(stderr, "===== Unhandled unknown error\n");
-    return HTTP500HtmlResponse(*this, request)
+    return HTTP500Response(*this, request)
          + "Unknown error";
   }
 }
@@ -439,21 +619,6 @@ SuggestionsList_t getSuggestions(SuggestionSearcherCache& cache, const zim::Arch
   return suggestions;
 }
 
-namespace
-{
-
-std::string noSuchBookErrorMsg(const std::string& bookName)
-{
-  return "No such book: " + bookName;
-}
-
-std::string noSearchResultsMsg()
-{
-  return "The fulltext search engine is not available for this content.";
-}
-
-} // unnamed namespace
-
 std::unique_ptr<Response> InternalServer::handle_suggest(const RequestContext& request)
 {
   if (m_verbose.load()) {
@@ -471,7 +636,7 @@ std::unique_ptr<Response> InternalServer::handle_suggest(const RequestContext& r
   }
 
   if (archive == nullptr) {
-    return HTTP404HtmlResponse(*this, request)
+    return HTTP404Response(*this, request)
            + noSuchBookErrorMsg(bookName)
            + TaskbarInfo(bookName);
   }
@@ -514,7 +679,8 @@ std::unique_ptr<Response> InternalServer::handle_suggest(const RequestContext& r
   /* Propose the fulltext search if possible */
   if (archive->hasFulltextIndex()) {
     MustacheData result;
-    result.set("label", "containing '" + queryString + "'...");
+    const auto lang = request.get_user_language();
+    result.set("label", makeFulltextSearchSuggestion(lang, queryString));
     result.set("value", queryString + " ");
     result.set("kind", "pattern");
     result.set("first", first);
@@ -543,7 +709,7 @@ std::unique_ptr<Response> InternalServer::handle_skin(const RequestContext& requ
     response->set_cacheable();
     return std::move(response);
   } catch (const ResourceNotFound& e) {
-    return HTTP404HtmlResponse(*this, request)
+    return HTTP404Response(*this, request)
            + urlNotFoundMsg;
   }
 }
@@ -555,60 +721,43 @@ std::unique_ptr<Response> InternalServer::handle_search(const RequestContext& re
   }
 
   try {
-    auto searchInfo = SearchInfo(request);
-
-    std::string bookId;
-    std::shared_ptr<zim::Archive> archive;
-    if (!searchInfo.bookName.empty()) {
-      try {
-        bookId = mp_nameMapper->getIdForName(searchInfo.bookName);
-        archive = mp_library->getArchiveById(bookId);
-      } catch (const std::out_of_range&) {
-        throw std::invalid_argument("The requested book doesn't exist.");
-      }
-    }
-
+    auto searchInfo = getSearchInfo(request);
+    auto bookIds = searchInfo.getBookIds();
 
     /* Make the search */
     // Try to get a search from the searchInfo, else build it
+    auto searcher = mp_library->getSearcherByIds(bookIds);
+    auto lock(searcher->getLock());
+
     std::shared_ptr<zim::Search> search;
     try {
       search = searchCache.getOrPut(searchInfo,
         [=](){
-          std::shared_ptr<zim::Searcher> searcher;
-          if (archive) {
-            searcher = searcherCache.getOrPut(bookId, [=](){ return std::make_shared<zim::Searcher>(*archive);});
-          } else {
-            for (auto& bookId: mp_library->filter(kiwix::Filter().local(true).valid(true))) {
-              auto currentArchive = mp_library->getArchiveById(bookId);
-              if (currentArchive) {
-                if (! searcher) {
-                  searcher = std::make_shared<zim::Searcher>(*currentArchive);
-                } else {
-                  searcher->addArchive(*currentArchive);
-                }
-              }
-           }
-          }
           return make_shared<zim::Search>(searcher->search(searchInfo.getZimQuery(m_verbose.load())));
         }
       );
     } catch(std::runtime_error& e) {
       // Searcher->search will throw a runtime error if there is no valid xapian database to do the search.
       // (in case of zim file not containing a index)
-      return HTTPErrorHtmlResponse(*this, request, MHD_HTTP_NOT_FOUND,
-                                   "Fulltext search unavailable",
-                                   "Not Found",
-                                   m_root + "/skin/search_results.css")
-           + noSearchResultsMsg()
-           + TaskbarInfo(searchInfo.bookName, archive.get());
+      const auto cssUrl = renderUrl(m_root, RESOURCE::templates::url_of_search_results_css);
+      HTTPErrorResponse response(*this, request, MHD_HTTP_NOT_FOUND,
+                                 "fulltext-search-unavailable",
+                                 "404-page-heading",
+                                 cssUrl);
+      response += nonParameterizedMessage("no-search-results");
+      if(bookIds.size() == 1) {
+        auto bookId = *bookIds.begin();
+        auto bookName = mp_nameMapper->getNameForId(bookId);
+        response += TaskbarInfo(bookName, mp_library->getArchiveById(bookId).get());
+      }
+      return response;
     }
 
-
-    auto start = 0;
+    auto start = 1;
     try {
       start = request.get_argument<unsigned int>("start");
     } catch (const std::exception&) {}
+    start = max(1, start);
 
     auto pageLength = 25;
     try {
@@ -622,24 +771,29 @@ std::unique_ptr<Response> InternalServer::handle_search(const RequestContext& re
     }
 
     /* Get the results */
-    SearchRenderer renderer(search->getResults(start, pageLength), mp_nameMapper, mp_library, start,
+    SearchRenderer renderer(search->getResults(start-1, pageLength), mp_nameMapper, mp_library, start,
                             search->getEstimatedMatches());
     renderer.setSearchPattern(searchInfo.pattern);
-    renderer.setSearchContent(searchInfo.bookName);
+    renderer.setSearchBookQuery(searchInfo.bookFilterQuery);
     renderer.setProtocolPrefix(m_root + "/");
-    renderer.setSearchProtocolPrefix(m_root + "/search?");
+    renderer.setSearchProtocolPrefix(m_root + "/search");
     renderer.setPageLength(pageLength);
+    if (request.get_requested_format() == "xml") {
+      return ContentResponse::build(*this, renderer.getXml(), "application/rss+xml; charset=utf-8",
+      /*isHomePage =*/false,
+      /*raw =*/true);
+    }
     auto response = ContentResponse::build(*this, renderer.getHtml(), "text/html; charset=utf-8");
-    response->set_taskbar(searchInfo.bookName, archive.get());
+    if(bookIds.size() == 1) {
+      auto bookId = *bookIds.begin();
+      auto bookName = mp_nameMapper->getNameForId(bookId);
+      response->set_taskbar(bookName, mp_library->getArchiveById(bookId).get());
+    }
     return std::move(response);
-  } catch (const std::invalid_argument& e) {
-    return HTTP400HtmlResponse(*this, request)
+  } catch (const Error& e) {
+    return HTTP400Response(*this, request)
       + invalidUrlMsg
-      + std::string(e.what());
-  } catch (const std::exception& e) {
-    std::cerr << e.what() << std::endl;
-    return HTTP500HtmlResponse(*this, request)
-         + e.what();
+      + e.message();
   }
 }
 
@@ -660,7 +814,7 @@ std::unique_ptr<Response> InternalServer::handle_random(const RequestContext& re
   }
 
   if (archive == nullptr) {
-    return HTTP404HtmlResponse(*this, request)
+    return HTTP404Response(*this, request)
            + noSuchBookErrorMsg(bookName)
            + TaskbarInfo(bookName);
   }
@@ -669,9 +823,8 @@ std::unique_ptr<Response> InternalServer::handle_random(const RequestContext& re
     auto entry = archive->getRandomEntry();
     return build_redirect(bookName, getFinalItem(*archive, entry));
   } catch(zim::EntryNotFound& e) {
-    const std::string error_details = "Oops! Failed to pick a random article :(";
-    return HTTP404HtmlResponse(*this, request)
-           + error_details
+    return HTTP404Response(*this, request)
+           + nonParameterizedMessage("random-article-failure")
            + TaskbarInfo(bookName, archive.get());
   }
 }
@@ -684,7 +837,7 @@ std::unique_ptr<Response> InternalServer::handle_captured_external(const Request
   } catch (const std::out_of_range& e) {}
 
   if (source.empty()) {
-    return HTTP404HtmlResponse(*this, request)
+    return HTTP404Response(*this, request)
            + urlNotFoundMsg;
   }
 
@@ -705,7 +858,7 @@ std::unique_ptr<Response> InternalServer::handle_catalog(const RequestContext& r
     host = request.get_header("Host");
     url  = request.get_url_part(1);
   } catch (const std::out_of_range&) {
-    return HTTP404HtmlResponse(*this, request)
+    return HTTP404Response(*this, request)
            + urlNotFoundMsg;
   }
 
@@ -714,7 +867,7 @@ std::unique_ptr<Response> InternalServer::handle_catalog(const RequestContext& r
   }
 
   if (url != "searchdescription.xml" && url != "root.xml" && url != "search") {
-    return HTTP404HtmlResponse(*this, request)
+    return HTTP404Response(*this, request)
            + urlNotFoundMsg;
   }
 
@@ -743,45 +896,6 @@ std::unique_ptr<Response> InternalServer::handle_catalog(const RequestContext& r
   return std::move(response);
 }
 
-namespace
-{
-
-Filter get_search_filter(const RequestContext& request)
-{
-    auto filter = kiwix::Filter().valid(true).local(true);
-    try {
-      filter.query(request.get_argument("q"));
-    } catch (const std::out_of_range&) {}
-    try {
-      filter.maxSize(request.get_argument<unsigned long>("maxsize"));
-    } catch (...) {}
-    try {
-      filter.name(request.get_argument("name"));
-    } catch (const std::out_of_range&) {}
-    try {
-      filter.category(request.get_argument("category"));
-    } catch (const std::out_of_range&) {}
-    try {
-      filter.lang(request.get_argument("lang"));
-    } catch (const std::out_of_range&) {}
-    try {
-      filter.acceptTags(kiwix::split(request.get_argument("tag"), ";"));
-    } catch (...) {}
-    try {
-      filter.rejectTags(kiwix::split(request.get_argument("notag"), ";"));
-    } catch (...) {}
-    return filter;
-}
-
-template<class T>
-std::vector<T> subrange(const std::vector<T>& v, size_t s, size_t n)
-{
-  const size_t e = std::min(v.size(), s+n);
-  return std::vector<T>(v.begin()+std::min(v.size(), s), v.begin()+e);
-}
-
-} // unnamed namespace
-
 std::vector<std::string>
 InternalServer::search_catalog(const RequestContext& request,
                                kiwix::OPDSDumper& opdsDumper)
@@ -812,13 +926,13 @@ std::string get_book_name(const RequestContext& request)
   }
 }
 
-std::string searchSuggestionHTML(const std::string& searchURL, const std::string& pattern)
+ParameterizedMessage suggestSearchMsg(const std::string& searchURL, const std::string& pattern)
 {
-  kainjow::mustache::mustache tmpl("Make a full text search for <a href=\"{{{searchURL}}}\">{{pattern}}</a>");
-  MustacheData data;
-  data.set("pattern", pattern);
-  data.set("searchURL", searchURL);
-  return (tmpl.render(data));
+  return ParameterizedMessage("suggest-search",
+                              {
+                                { "PATTERN",    pattern   },
+                                { "SEARCH_URL", searchURL }
+                              });
 }
 
 } // unnamed namespace
@@ -850,9 +964,9 @@ std::unique_ptr<Response> InternalServer::handle_content(const RequestContext& r
 
   if (archive == nullptr) {
     const std::string searchURL = m_root + "/search?pattern=" + kiwix::urlEncode(pattern, true);
-    return HTTP404HtmlResponse(*this, request)
+    return HTTP404Response(*this, request)
            + urlNotFoundMsg
-           + searchSuggestionHTML(searchURL, kiwix::urlDecode(pattern))
+           + suggestSearchMsg(searchURL, kiwix::urlDecode(pattern))
            + TaskbarInfo(bookName);
   }
 
@@ -884,9 +998,9 @@ std::unique_ptr<Response> InternalServer::handle_content(const RequestContext& r
       printf("Failed to find %s\n", urlStr.c_str());
 
     std::string searchURL = m_root + "/search?content=" + bookName + "&pattern=" + kiwix::urlEncode(pattern, true);
-    return HTTP404HtmlResponse(*this, request)
+    return HTTP404Response(*this, request)
            + urlNotFoundMsg
-           + searchSuggestionHTML(searchURL, kiwix::urlDecode(pattern))
+           + suggestSearchMsg(searchURL, kiwix::urlDecode(pattern))
            + TaskbarInfo(bookName, archive.get());
   }
 }
@@ -904,15 +1018,14 @@ std::unique_ptr<Response> InternalServer::handle_raw(const RequestContext& reque
      bookName = request.get_url_part(1);
      kind = request.get_url_part(2);
   } catch (const std::out_of_range& e) {
-    return HTTP404HtmlResponse(*this, request)
+    return HTTP404Response(*this, request)
            + urlNotFoundMsg;
   }
 
   if (kind != "meta" && kind!= "content") {
-    const std::string error_details = kind + " is not a valid request for raw content.";
-    return HTTP404HtmlResponse(*this, request)
+    return HTTP404Response(*this, request)
            + urlNotFoundMsg
-           + error_details;
+           + invalidRawAccessMsg(kind);
   }
 
   std::shared_ptr<zim::Archive> archive;
@@ -922,7 +1035,7 @@ std::unique_ptr<Response> InternalServer::handle_raw(const RequestContext& reque
   } catch (const std::out_of_range& e) {}
 
   if (archive == nullptr) {
-    return HTTP404HtmlResponse(*this, request)
+    return HTTP404Response(*this, request)
            + urlNotFoundMsg
            + noSuchBookErrorMsg(bookName);
   }
@@ -948,10 +1061,9 @@ std::unique_ptr<Response> InternalServer::handle_raw(const RequestContext& reque
     if (m_verbose.load()) {
       printf("Failed to find %s\n", itemPath.c_str());
     }
-    const std::string error_details = "Cannot find " + kind + " entry " + itemPath;
-    return HTTP404HtmlResponse(*this, request)
+    return HTTP404Response(*this, request)
            + urlNotFoundMsg
-           + error_details;
+           + rawEntryNotFoundMsg(kind, itemPath);
   }
 }
 
