@@ -69,7 +69,7 @@
 
     function viewPortToCount(){
         const zoom = Math.floor((( window.outerWidth - 10 ) / window.innerWidth) * 100);
-        return Math.floor(window.innerHeight/(3*zoom) + 1)*(window.innerWidth/(2.5*zoom) + 1);
+        return Math.floor((window.innerHeight/(3*zoom) + 1)*(window.innerWidth/(2.5*zoom) + 1));
     }
 
     function getInnerHtml(node, query) {
@@ -142,10 +142,21 @@
         }
     }
 
+    async function getMagnetLink(downloadLink) {
+        const magnetUrl = downloadLink + '.magnet';
+        const controller = new AbortController();
+        setTimeout(() => controller.abort(), 5000);
+        const magnetLink = await fetch(magnetUrl, { signal: controller.signal }).then(response => {
+            return response.ok ? response.text() : '';
+        }).catch((_error) => '');
+        return magnetLink;
+    }
+
     function insertModal(button) {
         const downloadLink = button.getAttribute('data-link');
-        button.addEventListener('click', (event) => {
+        button.addEventListener('click', async (event) => {
             event.preventDefault();
+            const magnetLink = await getMagnetLink(downloadLink);
             document.body.insertAdjacentHTML('beforeend', `<div class="modal-wrapper">
                 <div class="modal">
                     <div class="modal-heading">
@@ -171,25 +182,26 @@
                     <div class="modal-content">
                         <div class="modal-regular-download">
                             <a href="${downloadLink}" download>
-                                <img src="../skin/download.png" alt="direct download" />
+                                <img src="../skin/download.png?KIWIXCACHEID" alt="direct download" />
                                 <div>Direct</div>
                             </a>
                         </div>
                         <div class="modal-regular-download">
                             <a href="${downloadLink}.sha256" download>
-                                <img src="../skin/hash.png" alt="download hash" />
+                                <img src="../skin/hash.png?KIWIXCACHEID" alt="download hash" />
                                 <div>Sha256 hash</div>
                             </a>
                         </div>
-                        <div class="modal-regular-download">
-                            <a href="${downloadLink}.magnet" target="_blank">
-                                <img src="../skin/magnet.png" alt="download magnet" />
+                        ${magnetLink ? 
+                        `<div class="modal-regular-download">
+                            <a href="${magnetLink}" target="_blank">
+                                <img src="../skin/magnet.png?KIWIXCACHEID" alt="download magnet" />
                                 <div>Magnet link</div>
                             </a>
-                        </div>
+                        </div>` : ``}
                         <div class="modal-regular-download">
                             <a href="${downloadLink}.torrent" download>
-                                <img src="../skin/bittorrent.png" alt="download torrent" />
+                                <img src="../skin/bittorrent.png?KIWIXCACHEID" alt="download torrent" />
                                 <div>Torrent file</div>
                             </a>
                         </div>
