@@ -1,27 +1,11 @@
 
+#define CPPHTTPLIB_ZLIB_SUPPORT 1
 #include "./httplib.h"
 #include "gtest/gtest.h"
 
-#include "../include/manager.h"
-#include "../include/server.h"
-#include "../include/name_mapper.h"
-#include "../include/tools.h"
+#define SERVER_PORT 8001
+#include "server_testing_tools.h"
 
-using TestContextImpl = std::vector<std::pair<std::string, std::string> >;
-struct TestContext : TestContextImpl {
-  TestContext(const std::initializer_list<value_type>& il)
-    : TestContextImpl(il)
-  {}
-};
-
-std::ostream& operator<<(std::ostream& out, const TestContext& ctx)
-{
-  out << "Test context:\n";
-  for ( const auto& kv : ctx )
-    out << "\t" << kv.first << ": " << kv.second << "\n";
-  out << std::endl;
-  return out;
-}
 
 bool is_valid_etag(const std::string& etag)
 {
@@ -37,125 +21,6 @@ T1 concat(T1 a, const T2& b)
   return a;
 }
 
-typedef httplib::Headers Headers;
-
-Headers invariantHeaders(Headers headers)
-{
-  headers.erase("Date");
-  return headers;
-}
-
-// Output generated via mustache templates sometimes contains end-of-line
-// whitespace. This complicates representing the expected output of a unit-test
-// as C++ raw strings in editors that are configured to delete EOL whitespace.
-// A workaround is to put special markers (//EOLWHITESPACEMARKER) at the end
-// of such lines in the expected output string and remove them at runtime.
-// This is exactly what this function is for.
-std::string removeEOLWhitespaceMarkers(const std::string& s)
-{
-  const std::regex pattern("//EOLWHITESPACEMARKER");
-  return std::regex_replace(s, pattern, "");
-}
-
-
-class ZimFileServer
-{
-public: // types
-  typedef std::shared_ptr<httplib::Response>  Response;
-  typedef std::vector<std::string> FilePathCollection;
-
-public: // functions
-  ZimFileServer(int serverPort, std::string libraryFilePath);
-  ZimFileServer(int serverPort, const FilePathCollection& zimpaths, std::string indexTemplateString = "");
-  ~ZimFileServer();
-
-  Response GET(const char* path, const Headers& headers = Headers())
-  {
-    return client->Get(path, headers);
-  }
-
-  Response HEAD(const char* path, const Headers& headers = Headers())
-  {
-    return client->Head(path, headers);
-  }
-
-private:
-  void run(int serverPort, std::string indexTemplateString = "");
-
-private: // data
-  kiwix::Library library;
-  kiwix::Manager manager;
-  std::unique_ptr<kiwix::HumanReadableNameMapper> nameMapper;
-  std::unique_ptr<kiwix::Server> server;
-  std::unique_ptr<httplib::Client> client;
-};
-
-ZimFileServer::ZimFileServer(int serverPort, std::string libraryFilePath)
-: manager(&this->library)
-{
-  if ( kiwix::isRelativePath(libraryFilePath) )
-    libraryFilePath = kiwix::computeAbsolutePath(kiwix::getCurrentDirectory(), libraryFilePath);
-  manager.readFile(libraryFilePath, true, true);
-  run(serverPort);
-}
-
-ZimFileServer::ZimFileServer(int serverPort, const FilePathCollection& zimpaths, std::string indexTemplateString)
-: manager(&this->library)
-{
-  for ( const auto& zimpath : zimpaths ) {
-    if (!manager.addBookFromPath(zimpath, zimpath, "", false))
-      throw std::runtime_error("Unable to add the ZIM file '" + zimpath + "'");
-  }
-  run(serverPort, indexTemplateString);
-}
-
-void ZimFileServer::run(int serverPort, std::string indexTemplateString)
-{
-  const std::string address = "127.0.0.1";
-  nameMapper.reset(new kiwix::HumanReadableNameMapper(library, false));
-  server.reset(new kiwix::Server(&library, nameMapper.get()));
-  server->setRoot("ROOT");
-  server->setAddress(address);
-  server->setPort(serverPort);
-  server->setNbThreads(2);
-  server->setVerbose(false);
-  if (!indexTemplateString.empty()) {
-    server->setIndexTemplateString(indexTemplateString);
-  }
-
-  if ( !server->start() )
-    throw std::runtime_error("ZimFileServer failed to start");
-
-  client.reset(new httplib::Client(address, serverPort));
-}
-
-ZimFileServer::~ZimFileServer()
-{
-  server->stop();
-}
-
-class ServerTest : public ::testing::Test
-{
-protected:
-  std::unique_ptr<ZimFileServer>   zfs1_;
-
-  const int PORT = 8001;
-  const ZimFileServer::FilePathCollection ZIMFILES {
-    "./test/zimfile.zim",
-    "./test/poor.zim",
-    "./test/corner_cases.zim"
-  };
-
-protected:
-  void SetUp() override {
-    zfs1_.reset(new ZimFileServer(PORT, ZIMFILES));
-  }
-
-  void TearDown() override {
-    zfs1_.reset();
-  }
-};
-
 const bool WITH_ETAG = true;
 const bool NO_ETAG = false;
 
@@ -185,13 +50,10 @@ const ResourceCollection resources200Compressible{
   { WITH_ETAG, "/ROOT/skin/taskbar.css" },
   { WITH_ETAG, "/ROOT/skin/block_external.js" },
 
-  { NO_ETAG,   "/ROOT/catalog/root.xml" },
-  { NO_ETAG,   "/ROOT/catalog/searchdescription.xml" },
   { NO_ETAG,   "/ROOT/catalog/search" },
 
   { NO_ETAG,   "/ROOT/search?content=zimfile&pattern=a" },
 
-  { NO_ETAG,   "/ROOT/suggest?content=zimfile" },
   { NO_ETAG,   "/ROOT/suggest?content=zimfile&term=ray" },
 
   { NO_ETAG,   "/ROOT/catch/external?source=www.example.com" },
@@ -224,7 +86,10 @@ const ResourceCollection resources200Uncompressible{
   { WITH_ETAG, "/ROOT/corner_cases/-/empty.css" },
   { WITH_ETAG, "/ROOT/corner_cases/-/empty.js" },
 
-  // The title and creator are too small to be compressed
+  // The following url's responses are too small to be compressed
+  { NO_ETAG,   "/ROOT/catalog/root.xml" },
+  { NO_ETAG,   "/ROOT/catalog/searchdescription.xml" },
+  { NO_ETAG,   "/ROOT/suggest?content=zimfile" },
   { WITH_ETAG, "/ROOT/raw/zimfile/meta/Creator" },
   { WITH_ETAG, "/ROOT/raw/zimfile/meta/Title" },
 };
@@ -241,7 +106,7 @@ TEST(indexTemplateStringTest, emptyIndexTemplate) {
     "./test/corner_cases.zim"
   };
 
-  ZimFileServer zfs(PORT, ZIMFILES, "");
+  ZimFileServer zfs(PORT, /*withTaskbar=*/true, ZIMFILES, "");
   EXPECT_EQ(200, zfs.GET("/ROOT/")->status);
 }
 
@@ -252,7 +117,7 @@ TEST(indexTemplateStringTest, indexTemplateCheck) {
     "./test/corner_cases.zim"
   };
 
-  ZimFileServer zfs(PORT, ZIMFILES, "<!DOCTYPE html><head>"
+  ZimFileServer zfs(PORT, /*withTaskbar=*/true, ZIMFILES, "<!DOCTYPE html><head>"
       "<title>Welcome to kiwix library</title>"
     "</head>"
   "</html>");
@@ -272,9 +137,9 @@ TEST_F(ServerTest, 200)
 TEST_F(ServerTest, CompressibleContentIsCompressedIfAcceptable)
 {
   for ( const Resource& res : resources200Compressible ) {
-    const auto x = zfs1_->GET(res.url, { {"Accept-Encoding", "deflate"} });
+    const auto x = zfs1_->GET(res.url, { {"Accept-Encoding", "gzip"} });
     EXPECT_EQ(200, x->status) << res;
-    EXPECT_EQ("deflate", x->get_header_value("Content-Encoding")) << res;
+    EXPECT_EQ("gzip", x->get_header_value("Content-Encoding")) << res;
     EXPECT_EQ("Accept-Encoding", x->get_header_value("Vary")) << res;
   }
 }
@@ -282,17 +147,100 @@ TEST_F(ServerTest, CompressibleContentIsCompressedIfAcceptable)
 TEST_F(ServerTest, UncompressibleContentIsNotCompressed)
 {
   for ( const Resource& res : resources200Uncompressible ) {
-    const auto x = zfs1_->GET(res.url, { {"Accept-Encoding", "deflate"} });
+    const auto x = zfs1_->GET(res.url, { {"Accept-Encoding", "gzip"} });
     EXPECT_EQ(200, x->status) << res;
     EXPECT_EQ("", x->get_header_value("Content-Encoding")) << res;
   }
 }
 
+
+// Selects from text only the lines containing the specified (fixed string)
+// pattern
+std::string fgrep(const std::string& pattern, const std::string& text)
+{
+  std::istringstream iss(text);
+  std::string line;
+  std::string result;
+  while ( getline(iss, line) ) {
+    if ( line.find(pattern) != std::string::npos ) {
+      result += line + "\n";
+    }
+  }
+  return result;
+}
+
+TEST_F(ServerTest, CacheIdsOfStaticResources)
+{
+  typedef std::pair<std::string, std::string> UrlAndExpectedResult;
+  const std::vector<UrlAndExpectedResult> testData{
+    {
+      /* url */ "/ROOT/",
+R"EXPECTEDRESULT(      src="/ROOT/skin/jquery-ui/external/jquery/jquery.js?cacheid=1d85f0f3"
+      src="/ROOT/skin/jquery-ui/jquery-ui.min.js?cacheid=d927c2ff"
+      href="/ROOT/skin/jquery-ui/jquery-ui.min.css?cacheid=e1de77b3"
+      href="/ROOT/skin/jquery-ui/jquery-ui.theme.min.css?cacheid=2a5841f9"
+      href="/ROOT/skin/index.css?cacheid=1aca980a"
+        src: url("/ROOT/skin/fonts/Poppins.ttf?cacheid=af705837") format("truetype");
+          src: url("/ROOT/skin/fonts/Roboto.ttf?cacheid=84d10248") format("truetype");
+    <script src="/ROOT/skin/isotope.pkgd.min.js?cacheid=2e48d392" defer></script>
+    <script src="/ROOT/skin/iso6391To3.js?cacheid=ecde2bb3"></script>
+    <script type="text/javascript" src="/ROOT/skin/index.js?cacheid=31ffa1f7" defer></script>
+)EXPECTEDRESULT"
+    },
+    {
+      /* url */ "/ROOT/skin/index.js",
+R"EXPECTEDRESULT(                                <img src="../skin/download.png?cacheid=a39aa502" alt="direct download" />
+                                <img src="../skin/hash.png?cacheid=f836e872" alt="download hash" />
+                                <img src="../skin/magnet.png?cacheid=73b6bddf" alt="download magnet" />
+                                <img src="../skin/bittorrent.png?cacheid=4f5c6882" alt="download torrent" />
+)EXPECTEDRESULT"
+    },
+    {
+      /* url */ "/ROOT/zimfile/A/index",
+R"EXPECTEDRESULT(<link type="root" href="/ROOT"><link type="text/css" href="/ROOT/skin/jquery-ui/jquery-ui.min.css?cacheid=e1de77b3" rel="Stylesheet" />
+<link type="text/css" href="/ROOT/skin/jquery-ui/jquery-ui.theme.min.css?cacheid=2a5841f9" rel="Stylesheet" />
+<link type="text/css" href="/ROOT/skin/taskbar.css?cacheid=49365e9c" rel="Stylesheet" />
+<script type="text/javascript" src="/ROOT/skin/jquery-ui/external/jquery/jquery.js?cacheid=1d85f0f3" defer></script>
+<script type="text/javascript" src="/ROOT/skin/jquery-ui/jquery-ui.min.js?cacheid=d927c2ff" defer></script>
+<script type="text/javascript" src="/ROOT/skin/taskbar.js?cacheid=5982280c" defer></script>
+        <label for="kiwix_button_show_toggle"><img src="/ROOT/skin/caret.png?cacheid=22b942b4" alt=""></label>
+)EXPECTEDRESULT"
+    },
+    {
+      // Searching in a ZIM file without a full-text index returns
+      // a page rendered from static/templates/no_search_result_html
+      /* url */ "/ROOT/search?content=poor&pattern=whatever",
+R"EXPECTEDRESULT(    <link type="text/css" href="/ROOT/skin/search_results.css?cacheid=76d39c84" rel="Stylesheet" />
+  <link type="root" href="/ROOT"><link type="text/css" href="/ROOT/skin/jquery-ui/jquery-ui.min.css?cacheid=e1de77b3" rel="Stylesheet" />
+<link type="text/css" href="/ROOT/skin/jquery-ui/jquery-ui.theme.min.css?cacheid=2a5841f9" rel="Stylesheet" />
+<link type="text/css" href="/ROOT/skin/taskbar.css?cacheid=49365e9c" rel="Stylesheet" />
+<script type="text/javascript" src="/ROOT/skin/jquery-ui/external/jquery/jquery.js?cacheid=1d85f0f3" defer></script>
+<script type="text/javascript" src="/ROOT/skin/jquery-ui/jquery-ui.min.js?cacheid=d927c2ff" defer></script>
+<script type="text/javascript" src="/ROOT/skin/taskbar.js?cacheid=5982280c" defer></script>
+        <label for="kiwix_button_show_toggle"><img src="/ROOT/skin/caret.png?cacheid=22b942b4" alt=""></label>
+)EXPECTEDRESULT"
+    },
+  };
+
+  for ( const auto& urlAndExpectedResult : testData ) {
+    const std::string url = urlAndExpectedResult.first;
+    const std::string expectedResult = urlAndExpectedResult.second;
+    const TestContext ctx{ {"url", url} };
+    const auto r = zfs1_->GET(url.c_str());
+    EXPECT_EQ(r->body.find("KIWIXCACHEID"), std::string::npos) << ctx;
+    EXPECT_EQ(fgrep("/skin/", r->body), expectedResult) << ctx;
+  }
+}
+
 const char* urls400[] = {
   "/ROOT/search",
   "/ROOT/search?content=zimfile",
   "/ROOT/search?content=non-existing-book&pattern=asdfqwerty",
   "/ROOT/search?content=non-existing-book&pattern=asd<qwerty",
+  "/ROOT/search?books.name=non-exsitent-book&pattern=asd<qwerty",
+  "/ROOT/search?books.id=non-exsitent-id&pattern=asd<qwerty",
+  "/ROOT/search?books.filter.lang=unk&pattern=asd<qwerty",
+  "/ROOT/search?pattern=foo",
   "/ROOT/search?pattern"
 };
 
@@ -362,7 +310,7 @@ enum ExpectedResponseDataType
 
 // Operator overloading is used as a means of defining a mini-DSL for
 // defining test data in a concise way (see usage in
-// TEST_F(ServerTest, 404WithBodyTesting))
+// TEST_F(ServerTest, Http404HtmlError))
 ExpectedResponseData operator==(ExpectedResponseDataType t, std::string s)
 {
   switch (t)
@@ -411,11 +359,13 @@ public:
   std::string expectedResponse() const;
 
 private:
+  bool isTranslatedVersion() const;
   virtual std::string pageTitle() const;
   std::string pageCssLink() const;
   std::string hiddenBookNameInput() const;
   std::string searchPatternInput() const;
   std::string taskbarLinks() const;
+  std::string goToWelcomePageText() const;
 };
 
 std::string TestContentIn404HtmlResponse::expectedResponse() const
@@ -431,12 +381,12 @@ std::string TestContentIn404HtmlResponse::expectedResponse() const
 )FRAG",
 
     R"FRAG(
-  <link type="root" href="/ROOT"><link type="text/css" href="/ROOT/skin/jquery-ui/jquery-ui.min.css" rel="Stylesheet" />
-<link type="text/css" href="/ROOT/skin/jquery-ui/jquery-ui.theme.min.css" rel="Stylesheet" />
-<link type="text/css" href="/ROOT/skin/taskbar.css" rel="Stylesheet" />
-<script type="text/javascript" src="/ROOT/skin/jquery-ui/external/jquery/jquery.js" defer></script>
-<script type="text/javascript" src="/ROOT/skin/jquery-ui/jquery-ui.min.js" defer></script>
-<script type="text/javascript" src="/ROOT/skin/taskbar.js" defer></script>
+  <link type="root" href="/ROOT"><link type="text/css" href="/ROOT/skin/jquery-ui/jquery-ui.min.css?cacheid=e1de77b3" rel="Stylesheet" />
+<link type="text/css" href="/ROOT/skin/jquery-ui/jquery-ui.theme.min.css?cacheid=2a5841f9" rel="Stylesheet" />
+<link type="text/css" href="/ROOT/skin/taskbar.css?cacheid=49365e9c" rel="Stylesheet" />
+<script type="text/javascript" src="/ROOT/skin/jquery-ui/external/jquery/jquery.js?cacheid=1d85f0f3" defer></script>
+<script type="text/javascript" src="/ROOT/skin/jquery-ui/jquery-ui.min.js?cacheid=d927c2ff" defer></script>
+<script type="text/javascript" src="/ROOT/skin/taskbar.js?cacheid=5982280c" defer></script>
 </head>
   <body><span class="kiwix">
   <span id="kiwixtoolbar" class="ui-widget-header">
@@ -452,9 +402,13 @@ std::string TestContentIn404HtmlResponse::expectedResponse() const
   R"FRAG(        </form>
       </div>
         <input type="checkbox" id="kiwix_button_show_toggle">
-        <label for="kiwix_button_show_toggle"><img src="/ROOT/skin/caret.png" alt=""></label>
+        <label for="kiwix_button_show_toggle"><img src="/ROOT/skin/caret.png?cacheid=22b942b4" alt=""></label>
         <div class="kiwix_button_cont">
-            <a id="kiwix_serve_taskbar_library_button" title="Go to welcome page" aria-label="Go to welcome page" href="/ROOT/"><button>&#x1f3e0;</button></a>
+            <a id="kiwix_serve_taskbar_library_button" title=")FRAG",
+
+  R"FRAG(" aria-label=")FRAG",
+
+  R"FRAG(" href="/ROOT/"><button>&#x1f3e0;</button></a>
           )FRAG",
 
   R"FRAG(
@@ -478,10 +432,14 @@ std::string TestContentIn404HtmlResponse::expectedResponse() const
        + frag[3]
        + searchPatternInput()
        + frag[4]
-       + taskbarLinks()
+       + goToWelcomePageText()
        + frag[5]
-       + removeEOLWhitespaceMarkers(expectedBody)
-       + frag[6];
+       + goToWelcomePageText()
+       + frag[6]
+       + taskbarLinks()
+       + frag[7]
+       + expectedBody
+       + frag[8];
 }
 
 std::string TestContentIn404HtmlResponse::pageTitle() const
@@ -510,11 +468,15 @@ std::string TestContentIn404HtmlResponse::hiddenBookNameInput() const
 
 std::string TestContentIn404HtmlResponse::searchPatternInput() const
 {
-  return R"(          <input autocomplete="off" class="ui-autocomplete-input" id="kiwixsearchbox" name="pattern" type="text" title="Search ')"
-       + bookTitle
-       + R"('" aria-label="Search ')"
-       + bookTitle
-       + R"('">
+  const std::string searchboxTooltip = isTranslatedVersion()
+                                     ? "Որոնել '" + bookTitle + "'֊ում"
+                                     : "Search '" + bookTitle + "'";
+
+  return R"(          <input autocomplete="off" class="ui-autocomplete-input" id="kiwixsearchbox" name="pattern" type="text" title=")"
+       + searchboxTooltip
+       + R"(" aria-label=")"
+       + searchboxTooltip
+       + R"(">
 )";
 }
 
@@ -523,21 +485,46 @@ std::string TestContentIn404HtmlResponse::taskbarLinks() const
   if ( bookName.empty() )
     return "";
 
-  return R"(<a id="kiwix_serve_taskbar_home_button" title="Go to the main page of ')"
-       + bookTitle
-       + R"('" aria-label="Go to the main page of ')"
-       + bookTitle
-       + R"('" href="/ROOT/)"
+  const auto goToMainPageOfBook = isTranslatedVersion()
+                                ? "Դեպի '" + bookTitle + "'֊ի գլխավոր էջը"
+                                : "Go to the main page of '" + bookTitle + "'";
+
+  const std::string goToRandomPage = isTranslatedVersion()
+                                   ? "Բացել պատահական էջ"
+                                   : "Go to a randomly selected page";
+
+  return R"(<a id="kiwix_serve_taskbar_home_button" title=")"
+       + goToMainPageOfBook
+       + R"(" aria-label=")"
+       + goToMainPageOfBook
+       + R"(" href="/ROOT/)"
        + bookName
        + R"(/"><button>)"
        + bookTitle
        + R"(</button></a>
-          <a id="kiwix_serve_taskbar_random_button" title="Go to a randomly selected page" aria-label="Go to a randomly selected page"
+          <a id="kiwix_serve_taskbar_random_button" title=")"
+       + goToRandomPage
+       + R"(" aria-label=")"
+       + goToRandomPage
+       + R"("
             href="/ROOT/random?content=)"
        + bookName
        + R"("><button>&#x1F3B2;</button></a>)";
 }
 
+bool TestContentIn404HtmlResponse::isTranslatedVersion() const
+{
+  return url.find("userlang=hy") != std::string::npos;
+}
+
+std::string TestContentIn404HtmlResponse::goToWelcomePageText() const
+{
+  return isTranslatedVersion()
+       ? "Գրադարանի էջ"
+       : "Go to welcome page";
+}
+
+
 class TestContentIn400HtmlResponse : public TestContentIn404HtmlResponse
 {
 public:
@@ -556,10 +543,9 @@ std::string TestContentIn400HtmlResponse::pageTitle() const {
      : expectedPageTitle;
 }
 
-
 } // namespace TestingOfHtmlResponses
 
-TEST_F(ServerTest, 404WithBodyTesting)
+TEST_F(ServerTest, Http404HtmlError)
 {
   using namespace TestingOfHtmlResponses;
   const std::vector<TestContentIn404HtmlResponse> testData{
@@ -571,6 +557,15 @@ TEST_F(ServerTest, 404WithBodyTesting)
     </p>
 )"  },
 
+    { /* url */ "/ROOT/random?content=non-existent-book&userlang=hy",
+      expected_page_title=="Սխալ հասցե" &&
+      expected_body==R"(
+    <h1>Սխալ հասցե</h1>
+    <p>
+      Գիրքը բացակայում է՝ non-existent-book
+    </p>
+)"  },
+
     { /* url */ "/ROOT/suggest?content=no-such-book&term=whatever",
       expected_body==R"(
     <h1>Not Found</h1>
@@ -587,6 +582,15 @@ TEST_F(ServerTest, 404WithBodyTesting)
     </p>
 )"  },
 
+    { /* url */ "/ROOT/catalog/?userlang=hy",
+      expected_page_title=="Սխալ հասցե" &&
+      expected_body==R"(
+    <h1>Սխալ հասցե</h1>
+    <p>
+      Սխալ հասցե՝ /ROOT/catalog/
+    </p>
+)"  },
+
     { /* url */ "/ROOT/catalog/invalid_endpoint",
       expected_body==R"(
     <h1>Not Found</h1>
@@ -595,6 +599,15 @@ TEST_F(ServerTest, 404WithBodyTesting)
     </p>
 )"  },
 
+    { /* url */ "/ROOT/catalog/invalid_endpoint?userlang=hy",
+      expected_page_title=="Սխալ հասցե" &&
+      expected_body==R"(
+    <h1>Սխալ հասցե</h1>
+    <p>
+      Սխալ հասցե՝ /ROOT/catalog/invalid_endpoint
+    </p>
+)"  },
+
     { /* url */ "/ROOT/invalid-book/whatever",
       expected_body==R"(
     <h1>Not Found</h1>
@@ -643,6 +656,20 @@ TEST_F(ServerTest, 404WithBodyTesting)
     </p>
 )"  },
 
+    { /* url */ "/ROOT/zimfile/invalid-article?userlang=hy",
+      expected_page_title=="Սխալ հասցե" &&
+      book_name=="zimfile" &&
+      book_title=="Ray Charles" &&
+      expected_body==R"(
+    <h1>Սխալ հասցե</h1>
+    <p>
+      Սխալ հասցե՝ /ROOT/zimfile/invalid-article
+    </p>
+    <p>
+      Որոնել <a href="/ROOT/search?content=zimfile&pattern=invalid-article">invalid-article</a>
+    </p>
+)"  },
+
     { /* url */ "/ROOT/raw/no-such-book/meta/Title",
       expected_body==R"(
     <h1>Not Found</h1>
@@ -689,7 +716,7 @@ TEST_F(ServerTest, 404WithBodyTesting)
 
     { /* url */ "/ROOT/search?content=poor&pattern=whatever",
       expected_page_title=="Fulltext search unavailable" &&
-      expected_css_url=="/ROOT/skin/search_results.css" &&
+      expected_css_url=="/ROOT/skin/search_results.css?cacheid=76d39c84" &&
       book_name=="poor" &&
       book_title=="poor" &&
       expected_body==R"(
@@ -708,7 +735,7 @@ TEST_F(ServerTest, 404WithBodyTesting)
   }
 }
 
-TEST_F(ServerTest, 400WithBodyTesting)
+TEST_F(ServerTest, Http400HtmlError)
 {
   using namespace TestingOfHtmlResponses;
   const std::vector<TestContentIn400HtmlResponse> testData{
@@ -719,7 +746,7 @@ TEST_F(ServerTest, 400WithBodyTesting)
       The requested URL "/ROOT/search" is not a valid request.
     </p>
     <p>
-      No query provided.
+      Too many books requested (4) where limit is 3
     </p>
 )"  },
     { /* url */ "/ROOT/search?content=zimfile",
@@ -739,7 +766,7 @@ TEST_F(ServerTest, 400WithBodyTesting)
       The requested URL "/ROOT/search?content=non-existing-book&pattern=asdfqwerty" is not a valid request.
     </p>
     <p>
-      The requested book doesn't exist.
+      No such book: non-existing-book
     </p>
 )"  },
     { /* url */ "/ROOT/search?content=non-existing-book&pattern=a\"<script foo>",
@@ -749,20 +776,30 @@ TEST_F(ServerTest, 400WithBodyTesting)
       The requested URL "/ROOT/search?content=non-existing-book&pattern=a"&lt;script foo&gt;" is not a valid request.
     </p>
     <p>
-      The requested book doesn't exist.
+      No such book: non-existing-book
     </p>
 )"  },
     // There is a flaw in our way to handle query string, we cannot differenciate
     // between `pattern` and `pattern=`
-    { /* url */ "/ROOT/search?pattern",
+    { /* url */ "/ROOT/search?books.filter.lang=eng&pattern",
       expected_body==R"(
     <h1>Invalid request</h1>
     <p>
-      The requested URL "/ROOT/search?pattern=" is not a valid request.
+      The requested URL "/ROOT/search?books.filter.lang=eng&pattern=" is not a valid request.
     </p>
     <p>
       No query provided.
     </p>
+)"  },
+    { /* url */ "/ROOT/search?pattern=foo",
+      expected_body==R"(
+    <h1>Invalid request</h1>
+    <p>
+      The requested URL "/ROOT/search?pattern=foo" is not a valid request.
+    </p>
+    <p>
+      Too many books requested (4) where limit is 3
+    </p>
 )"  },
   };
 
@@ -774,6 +811,85 @@ TEST_F(ServerTest, 400WithBodyTesting)
   }
 }
 
+TEST_F(ServerTest, HttpXmlError)
+{
+  struct TestData
+  {
+    std::string url;
+    int expectedStatusCode;
+    std::string expectedXml;
+
+    std::string fullExpectedXml() const
+    {
+      return R"(<?xml version="1.0" encoding="UTF-8">)" + expectedXml;
+    }
+
+    TestContext ctx() const
+    {
+      return TestContext{ {"url", url} };
+    }
+  };
+
+  const std::vector<TestData> testData{
+    { /* url */ "/ROOT/search?format=xml",
+      /* HTTP status code */ 400,
+      /* expected response XML */ R"(
+<error>Invalid request</error>
+<detail>The requested URL "/ROOT/search?format=xml" is not a valid request.</detail>
+<detail>Too many books requested (4) where limit is 3</detail>
+)"  },
+    { /* url */ "/ROOT/search?format=xml&content=zimfile",
+      /* HTTP status code */ 400,
+      /* expected response XML */ R"(
+<error>Invalid request</error>
+<detail>The requested URL "/ROOT/search?content=zimfile&format=xml" is not a valid request.</detail>
+<detail>No query provided.</detail>
+)"  },
+    { /* url */ "/ROOT/search?format=xml&content=non-existing-book&pattern=asdfqwerty",
+      /* HTTP status code */ 400,
+      /* expected response XML */ R"(
+<error>Invalid request</error>
+<detail>The requested URL "/ROOT/search?content=non-existing-book&format=xml&pattern=asdfqwerty" is not a valid request.</detail>
+<detail>No such book: non-existing-book</detail>
+)"  },
+    { /* url */ "/ROOT/search?format=xml&content=non-existing-book&pattern=a\"<script foo>",
+      /* HTTP status code */ 400,
+      /* expected response XML */ R"(
+<error>Invalid request</error>
+<detail>The requested URL "/ROOT/search?content=non-existing-book&format=xml&pattern=a"&lt;script foo&gt;" is not a valid request.</detail>
+<detail>No such book: non-existing-book</detail>
+)"  },
+    // There is a flaw in our way to handle query string, we cannot differenciate
+    // between `pattern` and `pattern=`
+    { /* url */ "/ROOT/search?format=xml&books.filter.lang=eng&pattern",
+      /* HTTP status code */ 400,
+      /* expected response XML */ R"(
+<error>Invalid request</error>
+<detail>The requested URL "/ROOT/search?books.filter.lang=eng&format=xml&pattern=" is not a valid request.</detail>
+<detail>No query provided.</detail>
+)"  },
+    { /* url */ "/ROOT/search?format=xml&pattern=foo",
+      /* HTTP status code */ 400,
+      /* expected response XML */ R"(
+<error>Invalid request</error>
+<detail>The requested URL "/ROOT/search?format=xml&pattern=foo" is not a valid request.</detail>
+<detail>Too many books requested (4) where limit is 3</detail>
+)"  },
+    { /* url */ "/ROOT/search?format=xml&content=poor&pattern=whatever",
+      /* HTTP status code */ 404,
+      /* expected response XML */ R"(
+<error>Fulltext search unavailable</error>
+<detail>The fulltext search engine is not available for this content.</detail>
+)"  },
+  };
+
+  for ( const auto& t : testData ) {
+    const auto r = zfs1_->GET(t.url.c_str());
+    EXPECT_EQ(r->status, t.expectedStatusCode) << t.ctx();
+    EXPECT_EQ(r->body, t.fullExpectedXml()) << t.ctx();
+  }
+}
+
 TEST_F(ServerTest, 500)
 {
   const std::string expectedBody = R"(<!DOCTYPE html>
@@ -800,6 +916,79 @@ TEST_F(ServerTest, 500)
   EXPECT_EQ(r->body, expectedBody);
 }
 
+TEST_F(ServerTest, UserLanguageControl)
+{
+  struct TestData
+  {
+    const std::string url;
+    const std::string acceptLanguageHeader;
+    const std::string expectedH1;
+
+    operator TestContext() const
+    {
+      return TestContext{
+          {"url", url},
+          {"acceptLanguageHeader", acceptLanguageHeader},
+      };
+    }
+  };
+
+  const TestData testData[] = {
+    {
+      /*url*/ "/ROOT/zimfile/invalid-article",
+      /*Accept-Language:*/ "",
+      /* expected <h1> */ "Not Found"
+    },
+    {
+      /*url*/ "/ROOT/zimfile/invalid-article?userlang=en",
+      /*Accept-Language:*/ "",
+      /* expected <h1> */ "Not Found"
+    },
+    {
+      /*url*/ "/ROOT/zimfile/invalid-article?userlang=hy",
+      /*Accept-Language:*/ "",
+      /* expected <h1> */ "Սխալ հասցե"
+    },
+    {
+      /*url*/ "/ROOT/zimfile/invalid-article",
+      /*Accept-Language:*/ "*",
+      /* expected <h1> */ "Not Found"
+    },
+    {
+      /*url*/ "/ROOT/zimfile/invalid-article",
+      /*Accept-Language:*/ "hy",
+      /* expected <h1> */ "Սխալ հասցե"
+    },
+    {
+      // userlang query parameter takes precedence over Accept-Language
+      /*url*/ "/ROOT/zimfile/invalid-article?userlang=en",
+      /*Accept-Language:*/ "hy",
+      /* expected <h1> */ "Not Found"
+    },
+    {
+      // The value of the Accept-Language header is not currently parsed.
+      // In case of a comma separated list of languages (optionally weighted
+      // with quality values) the default (en) language is used instead.
+      /*url*/ "/ROOT/zimfile/invalid-article",
+      /*Accept-Language:*/ "hy;q=0.9, en;q=0.2",
+      /* expected <h1> */ "Not Found"
+    },
+  };
+
+  const std::regex h1Regex("<h1>(.+)</h1>");
+  for ( const auto& t : testData ) {
+    std::smatch h1Match;
+    Headers headers;
+    if ( !t.acceptLanguageHeader.empty() ) {
+      headers.insert({"Accept-Language", t.acceptLanguageHeader});
+    }
+    const auto r = zfs1_->GET(t.url.c_str(), headers);
+    std::regex_search(r->body, h1Match, h1Regex);
+    const std::string h1(h1Match[1]);
+    EXPECT_EQ(h1, t.expectedH1) << t;
+  }
+}
+
 TEST_F(ServerTest, RandomPageRedirectsToAnExistingArticle)
 {
   auto g = zfs1_->GET("/ROOT/random?content=zimfile");
@@ -896,7 +1085,7 @@ TEST_F(ServerTest, ETagIsTheSameAcrossHeadAndGet)
 
 TEST_F(ServerTest, DifferentServerInstancesProduceDifferentETags)
 {
-  ZimFileServer zfs2(PORT + 1, ZIMFILES);
+  ZimFileServer zfs2(SERVER_PORT + 1, /*withTaskbar=*/true, ZIMFILES);
   for ( const Resource& res : all200Resources() ) {
     if ( !res.etag_expected ) continue;
     const auto h1 = zfs1_->HEAD(res.url);
@@ -911,7 +1100,7 @@ TEST_F(ServerTest, CompressionInfluencesETag)
     if ( ! res.etag_expected ) continue;
     const auto g1 = zfs1_->GET(res.url);
     const auto g2 = zfs1_->GET(res.url, { {"Accept-Encoding", ""} } );
-    const auto g3 = zfs1_->GET(res.url, { {"Accept-Encoding", "deflate"} } );
+    const auto g3 = zfs1_->GET(res.url, { {"Accept-Encoding", "gzip"} } );
     const auto etag = g1->get_header_value("ETag");
     EXPECT_EQ(etag, g2->get_header_value("ETag"));
     EXPECT_NE(etag, g3->get_header_value("ETag"));
@@ -924,7 +1113,7 @@ TEST_F(ServerTest, ETagOfUncompressibleContentIsNotAffectedByAcceptEncoding)
     if ( ! res.etag_expected ) continue;
     const auto g1 = zfs1_->GET(res.url);
     const auto g2 = zfs1_->GET(res.url, { {"Accept-Encoding", ""} } );
-    const auto g3 = zfs1_->GET(res.url, { {"Accept-Encoding", "deflate"} } );
+    const auto g3 = zfs1_->GET(res.url, { {"Accept-Encoding", "gzip"} } );
     const auto etag = g1->get_header_value("ETag");
     EXPECT_EQ(etag, g2->get_header_value("ETag")) << res;
     EXPECT_EQ(etag, g3->get_header_value("ETag")) << res;
@@ -963,7 +1152,7 @@ std::string make_etag_list(const std::string& etag)
 
 TEST_F(ServerTest, IfNoneMatchRequestsWithMatchingETagResultIn304Responses)
 {
-  const char* const encodings[] = { "", "deflate" };
+  const char* const encodings[] = { "", "gzip" };
   for ( const Resource& res : all200Resources() ) {
     for ( const char* enc: encodings ) {
       if ( ! res.etag_expected ) continue;
@@ -1094,7 +1283,7 @@ TEST_F(ServerTest, RangeHasPrecedenceOverCompression)
 
   const Headers onlyRange{ {"Range", "bytes=123-456"} };
   Headers rangeAndCompression(onlyRange);
-  rangeAndCompression.insert({"Accept-Encoding", "deflate"});
+  rangeAndCompression.insert({"Accept-Encoding", "gzip"});
 
   const auto p1 = zfs1_->GET(url, onlyRange);
   const auto p2 = zfs1_->GET(url, rangeAndCompression);
@@ -1159,6 +1348,23 @@ R"EXPECTEDRESPONSE([
     //EOLWHITESPACEMARKER
   }
 ]
+)EXPECTEDRESPONSE"
+    },
+    { /* url: */ "/ROOT/suggest?content=zimfile&term=öld%20suñ",
+R"EXPECTEDRESPONSE([
+  {
+    "value" : "That Lucky Old Sun",
+    "label" : "That Lucky &lt;b&gt;Old&lt;/b&gt; &lt;b&gt;Sun&lt;/b&gt;",
+    "kind" : "path"
+      , "path" : "A/That_Lucky_Old_Sun"
+  },
+  {
+    "value" : "öld suñ ",
+    "label" : "containing &apos;öld suñ&apos;...",
+    "kind" : "pattern"
+    //EOLWHITESPACEMARKER
+  }
+]
 )EXPECTEDRESPONSE"
     },
     { /* url: */ "/ROOT/suggest?content=zimfile&term=abracadabra",
@@ -1182,6 +1388,17 @@ R"EXPECTEDRESPONSE([
     //EOLWHITESPACEMARKER
   }
 ]
+)EXPECTEDRESPONSE"
+    },
+    { /* url: */ "/ROOT/suggest?content=zimfile&term=abracadabra&userlang=hy",
+R"EXPECTEDRESPONSE([
+  {
+    "value" : "abracadabra ",
+    "label" : "որոնել &apos;abracadabra&apos;...",
+    "kind" : "pattern"
+    //EOLWHITESPACEMARKER
+  }
+]
 )EXPECTEDRESPONSE"
     },
   };
